@@ -4,10 +4,11 @@ import argparse
 import math
 
 import ezdxf
-import svgpathtools
 
 from ..calc import calc_distance  # pylint: disable=E0402
+from ..ext import svgpathtools
 from ..input_plugins_base import DrawReaderBase
+from ..vc_types import VcSegment
 
 
 class DrawReader(DrawReaderBase):
@@ -57,7 +58,7 @@ class DrawReader(DrawReaderBase):
                 # print("##path", path)
                 for segment in path:
                     if isinstance(segment, svgpathtools.path.Line):
-                        self.add_line(
+                        self._add_line(
                             (segment.start.real, height - segment.start.imag),
                             (segment.end.real, height - segment.end.imag),
                         )
@@ -69,13 +70,13 @@ class DrawReader(DrawReaderBase):
                         nump = int(segment.length() / 3) + 1
                         for point_n in range(0, nump):
                             pos = segment.point(point_n / nump)
-                            self.add_line(
+                            self._add_line(
                                 (last_x, height - last_y), (pos.real, height - pos.imag)
                             )
                             last_x = pos.real
                             last_y = pos.imag
 
-                        self.add_line(
+                        self._add_line(
                             (last_x, height - last_y),
                             (segment.end.real, height - segment.end.imag),
                         )
@@ -83,7 +84,7 @@ class DrawReader(DrawReaderBase):
                         last_y = segment.end.imag
 
                 if path.iscontinuous():
-                    self.add_line(
+                    self._add_line(
                         (last_x, height - last_y),
                         (path[0].start.real, height - path[0].start.imag),
                     )
@@ -133,18 +134,20 @@ class DrawReader(DrawReaderBase):
                 dist = calc_distance((start.x, start.y), (end.x, end.y))
                 if dist > self.MIN_DIST:
                     self.segments.append(
-                        {
-                            "type": "ARC",
-                            "object": None,
-                            "layer": layer,
-                            "start": (start.x, start.y),
-                            "end": (end.x, end.y),
-                            "bulge": bulge,
-                            "center": (
-                                center[0],
-                                center[1],
-                            ),
-                        }
+                        VcSegment(
+                            {
+                                "type": "ARC",
+                                "object": None,
+                                "layer": layer,
+                                "start": (start.x, start.y),
+                                "end": (end.x, end.y),
+                                "bulge": bulge,
+                                "center": (
+                                    center[0],
+                                    center[1],
+                                ),
+                            }
+                        )
                     )
                 angle += astep
 
@@ -158,53 +161,22 @@ class DrawReader(DrawReaderBase):
             dist = calc_distance((start.x, start.y), (end.x, end.y))
             if dist > self.MIN_DIST:
                 self.segments.append(
-                    {
-                        "type": "ARC",
-                        "object": None,
-                        "layer": layer,
-                        "start": (start.x, start.y),
-                        "end": (end.x, end.y),
-                        "bulge": bulge,
-                        "center": (
-                            center[0],
-                            center[1],
-                        ),
-                    }
+                    VcSegment(
+                        {
+                            "type": "ARC",
+                            "object": None,
+                            "layer": layer,
+                            "start": (start.x, start.y),
+                            "end": (end.x, end.y),
+                            "bulge": bulge,
+                            "center": (
+                                center[0],
+                                center[1],
+                            ),
+                        }
+                    )
                 )
 
-    def add_line(self, start, end, layer="0") -> None:
-        dist = calc_distance(start, end)
-        if dist > self.MIN_DIST:
-            self.segments.append(
-                {
-                    "type": "LINE",
-                    "object": None,
-                    "layer": layer,
-                    "start": start,
-                    "end": end,
-                    "bulge": 0.0,
-                }
-            )
-
-    def get_segments(self) -> list[dict]:
-        return self.segments
-
-    def get_minmax(self) -> list[float]:
-        return self.min_max
-
-    def get_size(self) -> list[float]:
-        return self.size
-
-    def draw(self, draw_function, user_data=()) -> None:
-        for segment in self.segments:
-            draw_function(segment["start"], segment["end"], *user_data)
-
-    def draw_3d(self):
-        pass
-
-    def save_tabs(self, tabs: list) -> None:
-        pass
-
     @staticmethod
     def suffix() -> list[str]:
         return ["svg"]
