@@ -11,6 +11,7 @@ from .calc import (
     lines_intersect,
     rotate_list,
     vertex2points,
+    vertex_data_cache,
 )
 
 
@@ -76,11 +77,12 @@ class PostProcessor:
 
 def machine_cmd_begin(project: dict, post: PostProcessor) -> None:
     """machine_cmd-header"""
-    post.comment("--------------------------------------------------")
-    post.comment("Generator: viaConstructor")
-    post.comment(f"Filename: {project['filename_draw']}")
-    post.comment(f"Tool-Mode: {project['setup']['maschine']['mode']}")
-    post.comment("--------------------------------------------------")
+    if project["setup"]["maschine"]["comments"]:
+        post.comment("--------------------------------------------------")
+        post.comment("Generator: viaConstructor")
+        post.comment(f"Filename: {project['filename_draw']}")
+        post.comment(f"Tool-Mode: {project['setup']['maschine']['mode']}")
+        post.comment("--------------------------------------------------")
     post.separation()
     post.program_start()
     post.unit("mm")
@@ -107,7 +109,8 @@ def machine_cmd_begin(project: dict, post: PostProcessor) -> None:
 def machine_cmd_end(project: dict, post: PostProcessor) -> None:
     """machine_cmd-footer"""
     post.separation()
-    post.comment("- end -")
+    if project["setup"]["maschine"]["comments"]:
+        post.comment("- end -")
     if project["setup"]["maschine"]["mode"] != "laser" and "Z" in project["axis"]:
         post.move(z_pos=project["setup"]["mill"]["fast_move_z"])
         post.spindle_off()
@@ -241,7 +244,6 @@ def segment2machine_cmd(
                     or "Z" not in project["axis"]
                 ):
                     post.spindle_cw(project["setup"]["tool"]["speed"], pause=0)
-
                 break
 
         post.arc_ccw(
@@ -357,7 +359,6 @@ def segment2machine_cmd(
                     or "Z" not in project["axis"]
                 ):
                     post.spindle_cw(project["setup"]["tool"]["speed"], pause=0)
-
                 break
 
         post.arc_cw(
@@ -420,7 +421,7 @@ def segment2machine_cmd(
 
 
 def get_nearest_free_object(
-    polylines, level: int, last_pos: list, milling: dict, is_pocket: bool
+    polylines, level: int, last_pos: list, milling: set, is_pocket: bool
 ) -> tuple:
     found: bool = False
     nearest_dist: Union[None, float] = None
@@ -433,6 +434,7 @@ def get_nearest_free_object(
             and offset.is_pocket == is_pocket
             and offset.setup["mill"]["active"]
         ):
+
             if offset.is_pocket and offset.setup["pockets"]["insideout"]:
                 pocket_filter = None
                 offset_num_pre = offset_num.split(".")[0]
@@ -445,8 +447,8 @@ def get_nearest_free_object(
                 if offset_num != pocket_filter:
                     continue
 
+            vertex_data = vertex_data_cache(offset)
             if offset.is_closed():
-                vertex_data = offset.vertex_data()
                 if offset.start:
                     point_num = found_next_offset_point(
                         (offset.start[0], offset.start[1]), offset
@@ -472,8 +474,7 @@ def get_nearest_free_object(
                             found = True
                         point_num += 1
             else:
-                # on open obejcts, test first and last point
-                vertex_data = offset.vertex_data()
+                # on open objects, test first and last point
                 if len(vertex_data) > 0 and len(vertex_data[0]) > 0:
                     dist = calc_distance(
                         last_pos, (vertex_data[0][0], vertex_data[1][0])
@@ -483,6 +484,7 @@ def get_nearest_free_object(
                         nearest_idx = offset_num
                         nearest_point = 0
                         found = True
+
                     dist = calc_distance(
                         last_pos, (vertex_data[0][-1], vertex_data[1][-1])
                     )
@@ -491,12 +493,13 @@ def get_nearest_free_object(
                         nearest_idx = offset_num
                         nearest_point = len(vertex_data[0]) - 1
                         found = True
+
     return (found, nearest_idx, nearest_point)
 
 
 def polylines2machine_cmd(project: dict, post: PostProcessor) -> str:
     """generates machine_cmd from polilines"""
-    milling: dict = {}
+    milling: set = set()
     last_pos: list = [0, 0]
     polylines = project["offsets"]
     machine_cmd_begin(project, post)
@@ -505,17 +508,15 @@ def polylines2machine_cmd(project: dict, post: PostProcessor) -> str:
     order = 0
     for level in range(project["maxOuter"], -1, -1):
         for is_pocket in (True, False):
-
             while True:
                 (found, nearest_idx, nearest_point) = get_nearest_free_object(
                     polylines, level, last_pos, milling, is_pocket
                 )
 
                 if found:
-                    milling[nearest_idx] = nearest_idx
+                    milling.add(nearest_idx)
                     polyline = polylines[nearest_idx]
-
-                    vertex_data = polyline.vertex_data()
+                    vertex_data = vertex_data_cache(polyline)
                     is_closed = polyline.is_closed()
                     max_depth = polyline.setup["mill"]["depth"]
 
@@ -524,9 +525,8 @@ def polylines2machine_cmd(project: dict, post: PostProcessor) -> str:
                     else:
                         polyline.setup["tabs"]["data"] = []
 
-                    points = vertex2points(vertex_data)
                     if is_closed:
-                        points = rotate_list(points, nearest_point)
+                        points = rotate_list(vertex2points(vertex_data), nearest_point)
                     elif nearest_point != 0:
                         # redir open line and reverse bulge
                         x_start = list(vertex_data[0])
@@ -539,6 +539,8 @@ def polylines2machine_cmd(project: dict, post: PostProcessor) -> str:
                         for num, point in enumerate(bulges):
                             bulges[num] = -bulges[num]
                         points = vertex2points((x_start, y_start, bulges))
+                    else:
+                        points = vertex2points(vertex_data)
 
                     helix_mode = polyline.setup["mill"]["helix_mode"]
 
@@ -551,29 +553,34 @@ def polylines2machine_cmd(project: dict, post: PostProcessor) -> str:
                     if is_closed:
                         obj_distance += calc_distance(point, points[0])
 
-                    post.separation()
-                    post.comment("--------------------------------------------------")
-                    post.comment(f"Level: {level}")
-                    post.comment(f"Order: {order}")
-                    post.comment(f"Object: {nearest_idx}")
-                    post.comment(f"Distance: {obj_distance}mm")
-                    post.comment(f"Closed: {is_closed}")
-                    post.comment(f"isPocket: {polyline.is_pocket}")
-                    if (
-                        project["setup"]["maschine"]["mode"] != "laser"
-                        and "Z" in project["axis"]
-                    ):
+                    if project["setup"]["maschine"]["comments"]:
+                        post.separation()
                         post.comment(
-                            f"Depth: {polyline.setup['mill']['depth']}mm / {polyline.setup['mill']['step']}mm"
+                            "--------------------------------------------------"
                         )
-                    post.comment(
-                        f"Tool-Diameter: {project['setup']['tool']['diameter']}mm"
-                    )
-                    if polyline.tool_offset:
+                        post.comment(f"Level: {level}")
+                        post.comment(f"Order: {order}")
+                        post.comment(f"Object: {nearest_idx}")
+                        post.comment(f"Distance: {obj_distance}mm")
+                        post.comment(f"Closed: {is_closed}")
+                        post.comment(f"isPocket: {polyline.is_pocket}")
+                        if (
+                            project["setup"]["maschine"]["mode"] != "laser"
+                            and "Z" in project["axis"]
+                        ):
+                            post.comment(
+                                f"Depth: {polyline.setup['mill']['depth']}mm / {polyline.setup['mill']['step']}mm"
+                            )
                         post.comment(
-                            f"Tool-Offset: {project['setup']['tool']['diameter'] / 2.0}mm {polyline.tool_offset}"
+                            f"Tool-Diameter: {project['setup']['tool']['diameter']}mm"
+                        )
+                        if polyline.tool_offset:
+                            post.comment(
+                                f"Tool-Offset: {project['setup']['tool']['diameter'] / 2.0}mm {polyline.tool_offset}"
+                            )
+                        post.comment(
+                            "--------------------------------------------------"
                         )
-                    post.comment("--------------------------------------------------")
 
                     if is_closed:
                         if (
@@ -600,7 +607,8 @@ def polylines2machine_cmd(project: dict, post: PostProcessor) -> str:
                             project["setup"]["maschine"]["mode"] != "laser"
                             and "Z" in project["axis"]
                         ):
-                            post.comment(f"- Depth: {depth}mm -")
+                            if project["setup"]["maschine"]["comments"]:
+                                post.comment(f"- Depth: {depth}mm -")
 
                         if not is_closed:
                             if (
@@ -693,6 +701,7 @@ def polylines2machine_cmd(project: dict, post: PostProcessor) -> str:
                     else:
                         last_pos = points[-1]
                     order += 1
+
                 else:
                     break
 
