@@ -77,7 +77,7 @@ def setup_defaults(_) -> dict:
                 "tooltip": _("Overcuting edges"),
             },
             "zero": {
-                "default": "original",
+                "default": "bottomLeft",
                 "type": "select",
                 "options": (
                     ("original", _("original")),
@@ -353,6 +353,12 @@ def setup_defaults(_) -> dict:
                 "title": _("Tool-Mode"),
                 "tooltip": _("Tool-Mode"),
             },
+            "comments": {
+                "default": True,
+                "type": "bool",
+                "title": _("Comments in output"),
+                "tooltip": _("add comments to output"),
+            },
             "postcommand": {
                 "default": "",
                 "type": "str",
