@@ -8,6 +8,7 @@ import math
 import os
 import re
 import sys
+import time
 from copy import deepcopy
 from functools import partial
 from pathlib import Path
@@ -76,6 +77,7 @@ from .machine_cmd import polylines2machine_cmd
 from .output_plugins.gcode_linuxcnc import PostProcessorGcodeLinuxCNC
 from .output_plugins.hpgl import PostProcessorHpgl
 from .setupdefaults import setup_defaults
+from .vc_types import VcSegment
 
 try:
     from OpenGL import GL
@@ -86,7 +88,7 @@ except ImportError:
 
 
 reader_plugins: dict = {}
-for reader in ("dxfread", "hpglread", "stlread", "svgread", "ttfread"):
+for reader in ("dxfread", "hpglread", "stlread", "svgread", "ttfread", "imgread"):
     try:
         drawing_reader = importlib.import_module(
             f".{reader}", "viaconstructor.input_plugins"
@@ -96,14 +98,26 @@ for reader in ("dxfread", "hpglread", "stlread", "svgread", "ttfread"):
         print(f"ERRO while loading input plugin {reader}: {reader_error}")
 
 
-LAYER_REGEX = re.compile(r"([a-zA-Z]{1,4}):\s*([+-]?([0-9]+([.][0-9]*)?|[.][0-9]+))")
+DEBUG = False
+TIMESTAMP = 0
 
 
+def debug(message):
+    global TIMESTAMP  # pylint: disable=W0603
+    if DEBUG:
+        now = time.time()
+        if TIMESTAMP == 0:
+            TIMESTAMP = now
+        print(round(now - TIMESTAMP, 1))
+        print(f"{message} ", end="", flush=True)
+        TIMESTAMP = now
+
+
+# i18n
 def no_translation(text):
     return text
 
 
-# i18n
 _ = no_translation
 lang = os.environ.get("LANGUAGE")
 if lang:
@@ -362,14 +376,16 @@ class GLWidget(QGLWidget):
                     elif self.selector_mode == "repair":
                         obj_idx = self.selection[2]
                         self.project["segments_org"].append(
-                            {
-                                "type": "LINE",
-                                "object": None,
-                                "layer": self.project["objects"][obj_idx]["layer"],
-                                "start": (self.selection[0], self.selection[1]),
-                                "end": (self.selection[4], self.selection[5]),
-                                "bulge": 0.0,
-                            }
+                            VcSegment(
+                                {
+                                    "type": "LINE",
+                                    "object": None,
+                                    "layer": self.project["objects"][obj_idx]["layer"],
+                                    "start": (self.selection[0], self.selection[1]),
+                                    "end": (self.selection[4], self.selection[5]),
+                                    "bulge": 0.0,
+                                }
+                            )
                         )
                         self.selection = ()
                         self.project["app"].prepare_segments()
@@ -509,6 +525,10 @@ class GLWidget(QGLWidget):
 class ViaConstructor:
     """viaconstructor main class."""
 
+    LAYER_REGEX = re.compile(
+        r"([a-zA-Z]{1,4}):\s*([+-]?([0-9]+([.][0-9]*)?|[.][0-9]+))"
+    )
+
     project: dict = {
         "setup_defaults": setup_defaults(_),
         "filename_draw": "",
@@ -547,9 +567,9 @@ class ViaConstructor:
                 for segment in obj["segments"]:
                     if segment["bulge"] == 0.0:
                         msp.add_line(
-                            segment["start"],
-                            segment["end"],
-                            dxfattribs={"layer": segment["layer"]},
+                            segment.start,
+                            segment.end,
+                            dxfattribs={"layer": segment.layer},
                         )
                     else:
                         (
@@ -558,22 +578,23 @@ class ViaConstructor:
                             end_angle,  # pylint: disable=W0612
                             radius,  # pylint: disable=W0612
                         ) = ezdxf.math.bulge_to_arc(
-                            segment["start"], segment["end"], segment["bulge"]
+                            segment.start, segment.end, segment.bulge
                         )
                         msp.add_arc(
                             center=center,
                             radius=radius,
                             start_angle=start_angle * 180 / math.pi,
                             end_angle=end_angle * 180 / math.pi,
-                            dxfattribs={"layer": segment["layer"]},
+                            dxfattribs={"layer": segment.layer},
                         )
             for vport in doc.viewports.get_config("*Active"):  # type: ignore
                 vport.dxf.grid_on = True
-                vport.dxf.center = (200, 200)
+            ezdxf.zoom.extents(msp)  # type: ignore
             doc.saveas(output_file)
         except Exception as error:  # pylint: disable=W0703
-            print(f"ERROR while saving sxf: {error}")
+            print(f"ERROR while saving dxf: {error}")
             return False
+
         return True
 
     def run_calculation(self) -> None:
@@ -581,6 +602,8 @@ class ViaConstructor:
         if not self.draw_reader:
             return
 
+        debug("run_calculation: centercalc")
+
         psetup: dict = self.project["setup"]
         min_max = objects2minmax(self.project["objects"])
         self.project["minMax"] = min_max
@@ -602,6 +625,8 @@ class ViaConstructor:
             )
         self.project["minMax"] = objects2minmax(self.project["objects"])
 
+        debug("run_calculation: offsets")
+
         # create toolpath from objects
         self.project["offsets"] = objects2polyline_offsets(
             psetup["tool"]["diameter"],
@@ -611,13 +636,18 @@ class ViaConstructor:
         )
 
         # create machine commands
+        debug("run_calculation: maschine_commands")
         output_plugin: Union[PostProcessorHpgl, PostProcessorGcodeLinuxCNC]
         if self.project["setup"]["maschine"]["plugin"] == "gcode_linuxcnc":
-            output_plugin = PostProcessorGcodeLinuxCNC()
+            output_plugin = PostProcessorGcodeLinuxCNC(
+                self.project["setup"]["maschine"]["comments"]
+            )
             self.project["suffix"] = output_plugin.suffix()
             self.project["axis"] = output_plugin.axis()
         elif self.project["setup"]["maschine"]["plugin"] == "hpgl":
-            output_plugin = PostProcessorHpgl()
+            output_plugin = PostProcessorHpgl(
+                self.project["setup"]["maschine"]["comments"]
+            )
             self.project["suffix"] = output_plugin.suffix()
             self.project["axis"] = output_plugin.axis()
         else:
@@ -625,26 +655,31 @@ class ViaConstructor:
                 f"ERROR: Unknown maschine output plugin: {self.project['setup']['maschine']['plugin']}"
             )
             sys.exit(1)
-
         self.project["machine_cmd"] = polylines2machine_cmd(self.project, output_plugin)
+        debug("run_calculation: update textwidget")
         if self.project["textwidget"]:
             self.project["textwidget"].clear()
             self.project["textwidget"].insertPlainText(self.project["machine_cmd"])
             self.project["textwidget"].verticalScrollBar().setValue(0)
 
+        debug("run_calculation: done")
+
     def _toolbar_flipx(self) -> None:
         mirror_objects(self.project["objects"], self.project["minMax"], vertical=True)
         self.project["minMax"] = objects2minmax(self.project["objects"])
+        self.udate_tabs_data()
         self.update_drawing()
 
     def _toolbar_flipy(self) -> None:
         mirror_objects(self.project["objects"], self.project["minMax"], horizontal=True)
         self.project["minMax"] = objects2minmax(self.project["objects"])
+        self.udate_tabs_data()
         self.update_drawing()
 
     def _toolbar_rotate(self) -> None:
         rotate_objects(self.project["objects"], self.project["minMax"])
         self.project["minMax"] = objects2minmax(self.project["objects"])
+        self.udate_tabs_data()
         self.update_drawing()
 
     def _toolbar_scale(self) -> None:
@@ -658,6 +693,7 @@ class ViaConstructor:
         ):
             scale_objects(self.project["objects"], float(scale))
             self.project["minMax"] = objects2minmax(self.project["objects"])
+            self.udate_tabs_data()
             self.update_drawing()
 
     def _toolbar_view_2d(self) -> None:
@@ -752,6 +788,8 @@ class ViaConstructor:
             self.main, "Load Drawing", "", "drawing (*.dxf)"
         )
         if name[0] and self.load_drawing(name[0]):
+            self.update_table()
+            self.global_changed(0)
             self.update_drawing()
             self.status_bar_message(f"{self.info} - load drawing..done ({name[0]})")
         else:
@@ -816,9 +854,12 @@ class ViaConstructor:
         if not self.draw_reader:
             return
 
+        debug("update_drawing: start")
         self.status_bar_message(f"{self.info} - calculate..")
         if not draw_only:
+            debug("update_drawing: run_calculation")
             self.run_calculation()
+            debug("update_drawing: run_calculation done")
         self.project["gllist"] = GL.glGenLists(1)
         GL.glNewList(self.project["gllist"], GL.GL_COMPILE)
         draw_grid(self.project)
@@ -829,13 +870,17 @@ class ViaConstructor:
             self.project["glwidget"]
             and self.project["glwidget"].selector_mode != "repair"
         ):
+            debug("update_drawing: draw_maschinecode_path")
             if not draw_maschinecode_path(self.project):
                 self.status_bar_message(
                     f"{self.info} - error while drawing maschine commands"
                 )
+            debug("update_drawing: draw_maschinecode_path done")
 
         if self.project["setup"]["view"]["object_ids"]:
+            debug("update_drawing: draw_object_ids")
             draw_object_ids(self.project)
+            debug("update_drawing: draw_object_ids done")
 
         selected = -1
         if (
@@ -844,9 +889,11 @@ class ViaConstructor:
         ):
             selected = self.project["glwidget"].selection[2]
 
+        debug("update_drawing: draw_object_edges")
         draw_object_edges(self.project, selected=selected)
         if self.project["setup"]["view"]["polygon_show"]:
             draw_object_faces(self.project)
+        debug("update_drawing: draw_object_edges done")
         GL.glEndList()
         self.info = f"{self.project['minMax'][2] - self.project['minMax'][0]}x{self.project['minMax'][3] - self.project['minMax'][1]}mm"
         if self.main:
@@ -957,12 +1004,17 @@ class ViaConstructor:
     def update_table(self) -> None:
         """update objects table."""
 
+        debug("update_table: clear")
         self.project["objmodel"].clear()
         self.project["objmodel"].setHorizontalHeaderLabels(["Object", "Value"])
         # self.project["objwidget"].header().setDefaultSectionSize(180)
         self.project["objwidget"].setModel(self.project["objmodel"])
         root = self.project["objmodel"].invisibleRootItem()
 
+        if len(self.project["objects"]) >= 50:
+            debug(f"update_table: too many objects: {len(self.project['objects'])}")
+            return
+        debug("update_table: loading")
         for obj_idx, obj in self.project["objects"].items():
             if obj.get("layer", "").startswith("BREAKS:") or obj.get(
                 "layer", ""
@@ -1046,6 +1098,7 @@ class ViaConstructor:
                         else:
                             print(f"Unknown setup-type: {entry['type']}")
         # self.project["objwidget"].expandAll()
+        debug("update_table: done")
 
     def update_starts(self) -> None:
         """update starts."""
@@ -1099,7 +1152,6 @@ class ViaConstructor:
 
     def global_changed(self, value) -> None:  # pylint: disable=W0613
         """global setup changed."""
-
         if self.project["status"] == "CHANGE":
             return
 
@@ -1140,6 +1192,9 @@ class ViaConstructor:
         if self.project["setup"]["mill"]["step"] >= 0.0:
             self.project["setup"]["mill"]["step"] = -0.05
 
+        if not self.draw_reader:
+            return
+
         self.project["segments"] = deepcopy(self.project["segments_org"])
         self.project["segments"] = clean_segments(self.project["segments"])
         for obj in self.project["objects"].values():
@@ -1471,11 +1526,29 @@ class ViaConstructor:
                 else:
                     print(f"Unknown setup-type: {entry['type']}")
 
-    def prepare_segments(self) -> None:
-        segments = deepcopy(self.project["segments_org"])
-        self.project["segments"] = clean_segments(segments)
-        self.project["objects"] = segments2objects(self.project["segments"])
+    def udate_tabs_data(self) -> None:
         self.project["tabs"]["data"] = []
+        for obj in self.project["objects"].values():
+            layer = obj.get("layer")
+            if layer.startswith("BREAKS:") or layer.startswith("_TABS"):
+                obj["setup"]["mill"]["active"] = False
+                for segment in obj["segments"]:
+                    self.project["tabs"]["data"].append(
+                        (
+                            (segment.start[0], segment.start[1]),
+                            (segment.end[0], segment.end[1]),
+                        )
+                    )
+
+    def prepare_segments(self) -> None:
+        debug("prepare_segments: copy")
+        segments = deepcopy(self.project["segments_org"])
+        debug("prepare_segments: clean_segments")
+        self.project["segments"] = clean_segments(segments)
+        debug("prepare_segments: segments2objects")
+        self.project["objects"] = segments2objects(self.project["segments"])
+
+        debug("prepare_segments: setup")
         for obj in self.project["objects"].values():
             obj["setup"] = {}
             for sect in ("tool", "mill", "pockets", "tabs"):
@@ -1485,17 +1558,8 @@ class ViaConstructor:
             if layer:
                 if layer.startswith("IGNORE:"):
                     obj["setup"]["mill"]["active"] = False
-                elif layer.startswith("BREAKS:") or layer.startswith("_TABS"):
-                    obj["setup"]["mill"]["active"] = False
-                    for segment in obj["segments"]:
-                        self.project["tabs"]["data"].append(
-                            (
-                                (segment["start"][0], segment["start"][1]),
-                                (segment["end"][0], segment["end"][1]),
-                            )
-                        )
                 elif layer.startswith("MILL:"):
-                    matches = LAYER_REGEX.findall(obj["layer"])
+                    matches = self.LAYER_REGEX.findall(obj["layer"])
                     if matches:
                         for match in matches:
                             cmd = match[0].upper()
@@ -1510,11 +1574,15 @@ class ViaConstructor:
                                 obj["setup"]["tool"]["rate_h"] = int(value)
                             elif cmd in ("FEEDZ", "FZ"):
                                 obj["setup"]["tool"]["rate_v"] = int(value)
-
+        debug("prepare_segments: udate_tabs_data")
+        self.udate_tabs_data()
+        debug("prepare_segments: find_tool_offsets")
         self.project["maxOuter"] = find_tool_offsets(self.project["objects"])
+        debug("prepare_segments: done")
 
     def load_drawing(self, filename: str) -> bool:
         # clean project
+        debug("load_drawing: cleanup")
         self.project["filename_draw"] = ""
         self.project["filename_machine_cmd"] = ""
         self.project["suffix"] = "ngc"
@@ -1535,6 +1603,7 @@ class ViaConstructor:
         self.draw_reader = None
 
         # find plugin
+        debug("load_drawing: start")
         suffix = filename.split(".")[-1].lower()
         for reader_plugin in reader_plugins.values():
             if suffix in reader_plugin.suffix():
@@ -1544,19 +1613,29 @@ class ViaConstructor:
                 break
 
         if self.draw_reader:
+            debug("load_drawing: get segments")
             self.project["segments_org"] = self.draw_reader.get_segments()
             self.project["filename_draw"] = filename
             self.project[
                 "filename_machine_cmd"
             ] = f"{'.'.join(self.project['filename_draw'].split('.')[:-1])}.{self.project['suffix']}"
+            debug("load_drawing: prepare_segments")
             self.prepare_segments()
+            debug("load_drawing: done")
             return True
 
+        # disable some options on big drawings for a better view
+        if len(self.project["objects"]) >= 50:
+            self.project["setup"]["view"]["path"] = "minimal"
+            self.project["setup"]["view"]["object_ids"] = False
+
         print(f"ERROR: can not load file: {filename}")
+        debug("load_drawing: error")
         return False
 
     def __init__(self) -> None:
         """viaconstructor main init."""
+        debug("main: startup")
         setproctitle.setproctitle("viaconstructor")  # pylint: disable=I1101
 
         # arguments
@@ -1585,28 +1664,14 @@ class ViaConstructor:
             type=str,
             default=None,
         )
-        # STL-Reader
-        parser.add_argument(
-            "-z", "--zslice", help="slice at postion z (stl)", type=str, default=None
-        )
-        # TTF-Reader
-        parser.add_argument(
-            "-t",
-            "--text",
-            help="text for the Truetype reader",
-            type=str,
-            default="ViaConstructor",
-        )
-        parser.add_argument(
-            "-th",
-            "--text-height",
-            help="text height for the Truetype reader",
-            type=float,
-            default=100,
-        )
+
+        for reader_plugin in reader_plugins.values():
+            reader_plugin.arg_parser(parser)
+
         self.args = parser.parse_args()
 
         # load setup
+        debug("main: load setup")
         self.project["setup"] = {}
         for sname in self.project["setup_defaults"]:
             self.project["setup"][sname] = {}
@@ -1617,6 +1682,7 @@ class ViaConstructor:
             self.setup_load(self.args.setup)
 
         # load drawing #
+        debug("main: load drawing")
         if self.args.filename and self.load_drawing(self.args.filename):
             # save and exit
             if self.args.dxf:
@@ -1631,6 +1697,7 @@ class ViaConstructor:
                 sys.exit(0)
 
         # gui #
+        debug("main: load gui")
         qapp = QApplication(sys.argv)
         self.project["window"] = QWidget()
         self.project["app"] = self
@@ -1704,6 +1771,7 @@ class ViaConstructor:
 
         self.main.resize(1600, 1200)
         self.main.show()
+        debug("main: gui ready")
         sys.exit(qapp.exec_())
 
 
